@@ -45,6 +45,9 @@ struct _ghttp_request
   char               *username;
   char               *password;
   char               *authtoken;
+  char               *proxy_username;
+  char               *proxy_password;
+  char               *proxy_authtoken;
 };
 
 static const char *basic_header = "Basic ";
@@ -103,6 +106,22 @@ ghttp_request_destroy(ghttp_request *a_request)
       free(a_request->authtoken);
       a_request->authtoken = NULL;
     }
+  /* destroy proxy authentication */
+  if (a_request->proxy_username)
+    {
+      free(a_request->proxy_username);
+      a_request->proxy_username = NULL;
+    }
+  if (a_request->proxy_password)
+    {
+      free(a_request->proxy_password);
+      a_request->proxy_password = NULL;
+    }
+  if (a_request->proxy_authtoken)
+    {
+      free(a_request->proxy_authtoken);
+      a_request->proxy_authtoken = NULL;
+    }
   if (a_request)
     free(a_request);
   return;
@@ -329,6 +348,14 @@ ghttp_prepare(ghttp_request *a_request)
 			 http_hdr_WWW_Authenticate,
 			 NULL);
     }
+  /* set the proxy authorization header */
+  if ((a_request->proxy_authtoken != NULL) &&
+      (strlen(a_request->proxy_authtoken) > 0))
+    {
+      http_hdr_set_value(a_request->req->headers,
+			 http_hdr_Proxy_Authorization,
+			 a_request->proxy_authtoken);
+    }
   http_req_prepare(a_request->req);
   return 0;
 }
@@ -647,3 +674,68 @@ ghttp_set_authinfo(ghttp_request *a_request,
 
   return 0;
 }
+
+
+int
+ghttp_set_proxy_authinfo(ghttp_request *a_request,
+			 const char *a_user,
+			 const char *a_pass)
+{
+  char *l_authtoken = NULL;
+  char *l_final_auth = NULL;
+  char *l_auth64 = NULL;
+  
+  /* check our args */
+  if (!a_request)
+    return -1;
+  /* if we have a NULL or zero length string in the username
+     or password field, blitz the authinfo */
+  if ((!a_user) || (strlen(a_user) < 1) ||
+      (!a_pass) || (strlen(a_pass)< 1))
+    {
+      if (a_request->proxy_username)
+	{
+	  free(a_request->proxy_username);
+	  a_request->proxy_username = NULL;
+	}
+      if (a_request->proxy_password)
+	{
+	  free(a_request->proxy_password);
+	  a_request->proxy_password = NULL;
+	}
+      if (a_request->proxy_authtoken)
+	{
+	  free(a_request->proxy_authtoken);
+	  a_request->proxy_authtoken = NULL;
+	}
+      return 0;
+    }
+  /* encode the string using base64.  Usernames and passwords
+     for basic authentication are encoded like this:
+     username:password
+     That's it.  Easy, huh?
+  */
+  /* enough for the trailing \0 and the : */
+  l_authtoken = malloc(strlen(a_user) + strlen(a_pass) + 2);
+  memset(l_authtoken, 0, (strlen(a_user) + strlen(a_pass) + 2));
+  sprintf(l_authtoken, "%s:%s", a_user, a_pass);
+  l_auth64 = http_base64_encode(l_authtoken);
+  if (!l_auth64)
+    {
+      free(l_authtoken);
+      return -1;
+    }
+  /* build the final header */
+  l_final_auth = malloc(strlen(l_auth64) + strlen(basic_header) + 1);
+  memset(l_final_auth, 0, (strlen(l_auth64) + strlen(basic_header) + 1));
+  strcat(l_final_auth, basic_header);
+  strcat(l_final_auth, l_auth64);
+  free(l_auth64);
+  free(l_authtoken);
+  /* copy the strings into the request */
+  a_request->proxy_username = strdup(a_user);
+  a_request->proxy_password = strdup(a_pass);
+  a_request->proxy_authtoken = l_final_auth;
+  
+  return 0;
+}
