@@ -231,6 +231,15 @@ ghttp_set_authinfo(ghttp_request *a_request,
 		   const char *a_pass);
 		   
 
+ /* Set your username/password pair for proxy
+  */
+ 
+int
+ghttp_set_proxy_authinfo(ghttp_request *a_request,
+			 const char *a_user,
+			 const char *a_pass);
+
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
