@@ -175,6 +175,11 @@ http_resp_read_headers(http_resp *a_resp, http_trans_conn *a_conn)
      end of the headers */
   while (*l_cur_ptr != '\r')
     l_cur_ptr++;
+  /* make sure to free the buffer if it's already allocated */
+  if (a_resp->reason_phrase) {
+    free(a_resp->reason_phrase);
+    a_resp->reason_phrase = NULL;
+  }
   /* allocate space for the reason phrase */
   a_resp->reason_phrase =
     malloc((l_cur_ptr - (&l_ptr[13])) + 1);
