@@ -47,14 +47,18 @@ http_uri_parse(char *a_string,
   /* check the parameters */
   if (a_string == NULL)
     goto ec;
-  a_uri->full = strdup(a_string);
+  if (a_uri) {
+    a_uri->full = strdup(a_string);
+  }
   l_start_string = strchr(a_string, ':');
   /* check to make sure that there was a : in the string */
   if (!l_start_string)
     goto ec;
-  a_uri->proto = (char *)malloc(l_start_string - a_string + 1);
-  memcpy(a_uri->proto, a_string, (l_start_string - a_string));
-  a_uri->proto[l_start_string - a_string] = '\0';
+  if (a_uri) {
+    a_uri->proto = (char *)malloc(l_start_string - a_string + 1);
+    memcpy(a_uri->proto, a_string, (l_start_string - a_string));
+    a_uri->proto[l_start_string - a_string] = '\0';
+  }
   /* check to make sure it starts with "http://" */
   if (memcmp(l_start_string, "://", 3) != 0)
     goto ec;
